import pickle

import tensorflow
from keras.engine.saving import load_model

import betting.plotting as plotting
from betting.betting_evaluator import evaluate_games_weekly, evaluate_baselines_by_gameweek
from betting.strategy import FixedReturn, FixedBet, KellyRatio, VarianceAdjusted, FixedBetMostLikely, \
    FixedBetLeastLikely, FixedBetAlwaysHome
from datasets import dataloading
from datasets.dataloading import separate_test_data
from models.expected_lineup import ExpectedLineUp
from models.soccerpowerindexmodel import SoccerPowerIndex
from models.sub_expected_lineup import SubExpectedLineup
from models.sub_soccerpowerindexmodel import SubSoccerPowerIndex

INIT_BANKROLL = 100
MIN_CONFIDENCE = 1.1
MIN_PROBABILITY = 0.3

SAVE_PLOTS = False

best_models = [
    {
        "name": "Expected Lineup",
        "path": "../experiment/best_models/expected-lineup/dims=32_opt=adam_act=relu_lr=0.0001_loss=categorical_crossentropy_run{}-FINAL-model",
        "features_dict": ExpectedLineUp.get_default_features(),
        "dataframe": False
    },
    {
        "name": "Sub-Expected Lineup",
        "path": "../experiment/best_models/sub-expected-lineup/dims=32_opt=adam_act=tanh_lr=0.0001_loss=categorical_crossentropy_run{}-FINAL-model",
        "features_dict": SubExpectedLineup.get_default_features(),
        "dataframe": False
    },
    {
        "name": "FC-pi",  # Fully Connected pi-football
        "path": "../experiment/best_models/soccer-power-index/dims=32_opt=adam_act=relu_lr=0.0001_loss=categorical_crossentropy_run{}-FINAL-model",
        "features_dict": SoccerPowerIndex.get_default_features(),
        "dataframe": True
    },
    {
        "name": "SUB-pi",  # pi-football with sub-networks
        "path": "../experiment/best_models/sub-pi-football/dims=32_opt=adam_act=relu_lr=0.0001_loss=categorical_crossentropy_run{}-FINAL-model",
        "features_dict": SubSoccerPowerIndex.get_default_features(),
        "dataframe": True
    }
]

if __name__ == "__main__":
    test_league_seasons = {
        "leagues": ["EPL"],
        "seasons": ["2016", "2017"]
    }
    pw_data = pickle.load(open("../models/data_files/all-data-peewee-n_5-player-list", "rb"))
    pw_data, pw_test_data = separate_test_data(pw_data, test_league_seasons)
    data, df_test_data = dataloading.load_dataframe_data(test_league_seasons, "../models/data_files/all-data-df")

    for test_model in best_models:
        strategy_bankrolls = {}
        for i in range(5):
            best_fixed_bet_models = dict.fromkeys(test_league_seasons["seasons"], None)
            model = load_model(
                test_model["path"].format(i),
                custom_objects={"GlorotUniform": tensorflow.keras.initializers.glorot_uniform})
            print("Model # " + str(i))
            strategies = [
                FixedBet(INIT_BANKROLL, MIN_CONFIDENCE, MIN_PROBABILITY, bet_size=1),
                FixedReturn(INIT_BANKROLL, MIN_CONFIDENCE, MIN_PROBABILITY, expected_return=1),
                KellyRatio(INIT_BANKROLL, MIN_CONFIDENCE, MIN_PROBABILITY),
                VarianceAdjusted(INIT_BANKROLL, MIN_CONFIDENCE, MIN_PROBABILITY)
            ]

            if test_model["dataframe"]:
                _, model_strategy_bankroll_histories, model_strategy_bet_histories, prob_outcome_dists = evaluate_games_weekly(
                    model, test_league_seasons, df_test_data,
                    test_model["features_dict"], strategies,
                    dataframe=True,
                    one_bet_only=True)
            else:
                _, model_strategy_bankroll_histories, model_strategy_bet_histories, prob_outcome_dists = evaluate_games_weekly(
                    model, test_league_seasons, pw_test_data,
                    test_model["features_dict"], strategies,
                    dataframe=False,
                    one_bet_only=True)
            # plotting.plot_prediction_success_all_preds(test_model["name"], prob_outcome_dists)
            for season, strategies in model_strategy_bankroll_histories.items():
                if season not in strategy_bankrolls:
                    strategy_bankrolls[season] = {}
                for name, results in strategies.items():
                    if name not in strategy_bankrolls[season]:
                        strategy_bankrolls[season][name] = [results]
                    else:
                        strategy_bankrolls[season][name].append(results)

        plotting.plot_model_results(strategy_bankrolls, test_model["name"], save_plot=SAVE_PLOTS)

